"""
Template

"""
import logging
import WebFrame as WF
from WebFrame import auth
from WebFrame import Translate
#from jinja2.sandbox import SandboxedEnvironment as JE # Seems buggy
try:
    from jinja2.environment import Environment as JE
    from jinja2 import FileSystemLoader
except ImportError:
    print "Can't import jinja2, did you install it?"
    pass # Make pydoc happy if jinja2 isn't installed

# Global namespace for templates
__global_ns = {}
__global_funcs = {}

class _FuncWrapper:
    """Function wrapper before feeding them to template"""
    def __init__(self, fn, func, read_policy, write_policy):
        self.__ctx = None
        self.__fn = fn
        self.__func = func
        self.__read_policy = read_policy
        self.__write_policy = write_policy
    def set_ctx(self, ctx): self.__ctx = ctx
    def run(self, *args):
        if WF.get_opt("debug"):
            logging.debug("Checking access for function '%s'", self.__fn)
        if auth.check_access(self.__ctx, self.__read_policy,
                             self.__write_policy):
            if WF.get_opt("debug"):
                logging.debug("Template called %s%s", self.__fn, args)
            return self.__func(self.__ctx, *args)
    def func(self):
        return self.run
        



class Template:
    """Mostly for internal use"""
    def __init__(self, ctx):
        self.ctx = ctx
        self._ns = {}
        self._funcs = {}
        
        enc = WF.get_opt("file.default_encoding")
        self.__env = JE(loader=FileSystemLoader(ctx.req.docroot,
                                                encoding=enc))

        # Get query parameters
        self._ns["QUERY_PARAMS"] = self.ctx.req.params

        # Get path info
        pi = self.ctx.req.path_info
        if pi != None:
            self._ns["PATH_INFO"] = pi
            self._ns["PATH_PARTS"] = pi.split("/")
        
    def register_value(self, name, value):
        """Register a variable or constant for use in templates for
        this request."""
        self._ns[name] = value

    def register_func(self, name, func):
        """Register a function for use in templates for this request."""
        self._funcs[name] = _FuncWrapper(name, func, read_policy, write_policy)

    def _get_env(self):
        return self.__env


def register_value(name, value):
    """Register a variable or constant for use in ALL templates
    This is for global functions for this app, not for
    request-specific bindings.
    """
    __global_ns[name] = value

def register_func(name, func, read_policy, write_policy):
    """Register a function for use in ALL templates
    """
    __global_funcs[name] = _FuncWrapper(name, func, read_policy, write_policy)

def run_template(ctx, path, vars_={}, funcs={}, translate=True):
    """Run a template from a given path optionally specifying extra
    variables and functions.
    Vars and functions are specified as dicts.
    Please remember that no access checking is done on functions
    provided on the run_template call"""

    # Build namespace from app-globals, and request-globals
    ns = dict(__global_ns)
    ns.update(ctx.template._ns)
    ns.update(vars_)

    # Add session
    ns["SESSION"] = ctx.sess;

    # Add user, if we have one
    if ctx.user != None:
        ns["USER"] = ctx.user;

    # Insert functions, mapped to current request
    fs = __global_funcs
    fs.update(ctx.template._funcs)
    fs.update(funcs)
    for fn in fs:
        w = fs[fn]
        w.set_ctx(ctx)
        ns[fn] = w.func()

    # Add access check function
    ns["CHECK_ACCESS"] = lambda read,write: auth.check_access(ctx, read, write)
    
    # Add a reference to the namespace
    if WF.get_opt("template.export_ns"):
        ns["__ns"] = ns

    # Remove docroot from path
    if path.startswith(ctx.req.docroot):
        path = path[len(ctx.req.docroot):]
    env = ctx.template._get_env()
    tmpl = env.get_template(path)

    # Run the template, and optionally translate output
    out = tmpl.render(ns)
    if translate:
        out = Translate.translate(ctx, out)
    return out
